@@ -15,13 +15,14 @@ def _aamp_motifs(
     T,
     P,
     T_subseq_isfinite,
-    T_squared,
+    p,
     excl_zone,
     min_neighbors,
     max_distance,
     cutoff,
     max_matches,
     max_motifs,
+    atol=1e-8,
 ):
     """
     Find the top non-normalized motifs (i.e., without z-normalization) for time series
@@ -43,8 +44,8 @@ def _aamp_motifs(
         A boolean array that indicates whether a subsequence in `T` contains a
         `np.nan`/`np.inf` value (False)
 
-    T_squared : numpy.ndarray
-        Squared time series or sequence
+    p : float
+        The p-norm to apply for computing the Minkowski distance.
 
     excl_zone : int
         Size of the exclusion zone
@@ -72,6 +73,10 @@ def _aamp_motifs(
     max_motifs : int
         The maximum number of motifs to return.
 
+    atol : float, default 1e-8
+        The absolute tolerance parameter. This value will be added to `max_distance`
+        when comparing distances between subsequences.
+
     Return
     ------
     motif_distances : numpy.ndarray
@@ -94,7 +99,7 @@ def _aamp_motifs(
     candidate_idx = np.argmin(P[-1])
     while len(motif_indices) < max_motifs:
         profile_value = P[-1, candidate_idx]
-        if profile_value > cutoff:  # pragma: no cover
+        if profile_value > cutoff or not np.isfinite(profile_value):  # pragma: no cover
             break
 
         # If max_distance is a constant (independent of the distance profile D of Q
@@ -112,6 +117,9 @@ def _aamp_motifs(
             T,
             max_matches=None,
             max_distance=max_distance,
+            atol=atol,
+            query_idx=candidate_idx,
+            p=p,
         )
 
         if len(query_matches) > min_neighbors:
@@ -119,7 +127,7 @@ def _aamp_motifs(
             motif_indices.append(query_matches[:max_matches, 1])
 
         for idx in query_matches[:, 1]:
-            core.apply_exclusion_zone(P, int(idx), excl_zone)
+            core.apply_exclusion_zone(P, int(idx), excl_zone, np.inf)
 
         candidate_idx = np.argmin(P[-1])
 
@@ -141,6 +149,8 @@ def aamp_motifs(
     cutoff=None,
     max_matches=10,
     max_motifs=1,
+    atol=1e-8,
+    p=2.0,
 ):
     """
     Discover the top non-normalized motifs (i.e., without z-normalization) for time
@@ -150,6 +160,26 @@ def aamp_motifs(
     number of other subsequence matches in `T` (outside the exclusion zone) with a
     distance less or equal to `max_distance`.
 
+    Note that, in the best case scenario, the returned arrays would have shape
+    `(max_motifs, max_matches)` and contain all finite values. However, in reality,
+    many conditions (see below) need to be satisfied in order for this to be true. Any
+    truncation in the number of rows (i.e., motifs)  may be the result of insufficient
+    candidate motifs with matches greater than or equal to `min_neighbors` or that the
+    matrix profile value for the candidate motif was larger than `cutoff`. Similarly,
+    any truncationin in the number of columns (i.e., matches) may be the result of
+    insufficient matches being found with distances (to their corresponding candidate
+    motif) that are equal to or less than `max_distance`. Only motifs and matches that
+    satisfy all of these constraints will be returned.
+
+    If you must return a shape of `(max_motifs, max_matches)`, then you may consider
+    specifying a smaller `min_neighors`, a larger `max_distance`, and/or a larger
+    `cutoff`. For example, while it is ill advised, setting `min_neighbors=1`,
+    `max_distance=np.inf`, and `cutoff=np.inf` will ensure that the shape of the output
+    arrays will be `(max_motifs, max_matches)`. However, given the lack of constraints,
+    the quality of each motif and the quality of each match may be drastically
+    different. Setting appropriate conditions will help ensure appropriately
+    constrained results that may be easier to interpret.
+
     Parameters
     ----------
     T : numpy.ndarray
@@ -188,6 +218,13 @@ def aamp_motifs(
     max_motifs : int, default 1
         The maximum number of motifs to return.
 
+    atol : float, default 1e-8
+        The absolute tolerance parameter. This value will be added to `max_distance`
+        when comparing distances between subsequences.
+
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
+
     Return
     ------
     motif_distances : numpy.ndarray
@@ -217,7 +254,7 @@ def aamp_motifs(
 
     if P.ndim != 1:  # pragma: no cover
         raise ValueError(
-            f"T is {P.ndim}-dimensional and must be 1-dimensional. "
+            f"P is {P.ndim}-dimensional and must be 1-dimensional. "
             "Multidimensional motif discovery is not yet supported."
         )
 
@@ -233,20 +270,20 @@ def aamp_motifs(
         )
 
     T, T_subseq_isfinite = core.preprocess_non_normalized(T[np.newaxis, :], m)
-    T_squared = np.sum(core.rolling_window(T * T, m), axis=-1)
     P = P[np.newaxis, :].astype(np.float64)
 
     motif_distances, motif_indices = _aamp_motifs(
         T,
         P,
         T_subseq_isfinite,
-        T_squared,
+        p,
         excl_zone,
         min_neighbors,
         max_distance,
         cutoff,
         max_matches,
         max_motifs,
+        atol=atol,
     )
 
     return motif_distances, motif_indices
@@ -256,9 +293,11 @@ def aamp_match(
     Q,
     T,
     T_subseq_isfinite=None,
-    T_squared=None,
     max_distance=None,
     max_matches=None,
+    atol=1e-8,
+    query_idx=None,
+    p=2.0,
 ):
     """
     Find all matches of a query `Q` in a time series `T`, i.e. the indices
@@ -275,6 +314,10 @@ def aamp_match(
     T : numpy.ndarray
         The time series of interest
 
+    T_subseq_isfinite : numpy.ndarray, default None
+        A boolean array that indicates whether a subsequence in `T` contains a
+        `np.nan`/`np.inf` value (False)
+
     max_distance : float or function, default None
         Maximum distance between `Q` and a subsequence `S` for `S` to be considered a
         match. If a function, then it has to be a function of one argument `D`, which
@@ -289,12 +332,27 @@ def aamp_match(
         indices of the most similar `10` subsequences is returned. If `None`, then all
         occurrences are returned.
 
+    atol : float, default 1e-8
+        The absolute tolerance parameter. This value will be added to `max_distance`
+        when comparing distances between subsequences.
+
+    query_idx : int, default None
+        This is the index position along the time series, `T`, where the query
+        subsequence, `Q`, is located.
+        `query_idx` should only be used when the matrix profile is a self-join and
+        should be set to `None` for matrix profiles computed from AB-joins.
+        If `query_idx` is set to a specific integer value, then this will help ensure
+        that the self-match will be returned first.
+
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
+
     Returns
     -------
     out : numpy.ndarray
         The first column consists of distances of subsequences of `T` whose distances
-        to `Q` are smaller than `max_distance`, sorted by distance (lowest to highest).
-        The second column consists of the corresponding indices in `T`.
+        to `Q` are less than or equal to`max_distance`, sorted by distance (lowest to
+        highest). The second column consists of the corresponding indices in `T`.
     """
     if len(Q.shape) == 1:
         Q = Q[np.newaxis, :]
@@ -320,25 +378,31 @@ def aamp_match(
                 [np.nanmean(D_copy) - 2.0 * np.nanstd(D_copy), np.nanmin(D_copy)]
             )
 
-    if T_subseq_isfinite is None or T_squared is None:
+    if T_subseq_isfinite is None:
         T, T_subseq_isfinite = core.preprocess_non_normalized(T, m)
-        T_squared = np.sum(core.rolling_window(T * T, m), axis=-1)
 
-    D = [
-        core.mass_absolute(Q[i], T[i], T_subseq_isfinite[i], T_squared[i])
-        for i in range(d)
-    ]
+    D = np.empty((d, n - m + 1))
+    for i in range(d):
+        D[i, :] = core.mass_absolute(Q[i], T[i], T_subseq_isfinite[i], p=p)
 
-    D = np.sum(D, axis=0) / d
+    D = np.mean(D, axis=0)
     if not isinstance(max_distance, float):
         max_distance = max_distance(D)
 
     matches = []
 
-    candidate_idx = np.argmin(D)
-    while D[candidate_idx] <= max_distance and len(matches) < max_matches:
+    if query_idx is not None:
+        candidate_idx = query_idx
+    else:
+        candidate_idx = np.argmin(D)
+
+    while (
+        D[candidate_idx] <= atol + max_distance
+        and np.isfinite(D[candidate_idx])
+        and len(matches) < max_matches
+    ):
         matches.append([D[candidate_idx], candidate_idx])
-        core.apply_exclusion_zone(D, candidate_idx, excl_zone)
+        core.apply_exclusion_zone(D, candidate_idx, excl_zone, np.inf)
         candidate_idx = np.argmin(D)
 
     return np.array(matches, dtype=object)
