@@ -16,6 +16,7 @@ def _get_all_aampdist_profiles(
     mpdist_percentage=0.05,
     mpdist_k=None,
     mpdist_custom_func=None,
+    p=2.0,
 ):
     """
     For each non-overlapping subsequence, `S[i]`, in `T`, compute the matrix profile
@@ -60,6 +61,9 @@ def _get_all_aampdist_profiles(
         `MPdist` value. The `percentage` and `k` parameters are ignored when
         `mpdist_custom_func` is not None.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
+
     Returns
     -------
     D : numpy.ndarray
@@ -106,6 +110,7 @@ def _get_all_aampdist_profiles(
             percentage=mpdist_percentage,
             k=mpdist_k,
             custom_func=mpdist_custom_func,
+            p=p,
         )
 
     stop_idx = n_padded - m + 1 - right_pad
@@ -122,6 +127,7 @@ def aampdist_snippets(
     s=None,
     mpdist_percentage=0.05,
     mpdist_k=None,
+    p=2.0,
 ):
     """
     Identify the top `k` snippets that best represent the time series, `T`
@@ -160,6 +166,9 @@ def aampdist_snippets(
         Specify the `k`th value in the concatenated matrix profiles to return. When
         `mpdist_k` is not `None`, then the `mpdist_percentage` parameter is ignored.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
+
     Returns
     -------
     snippets : numpy.ndarray
@@ -206,6 +215,7 @@ def aampdist_snippets(
         s=s,
         mpdist_percentage=mpdist_percentage,
         mpdist_k=mpdist_k,
+        p=p,
     )
 
     pad_width = (0, int(m * np.ceil(T.shape[0] / m) - T.shape[0]))
