@@ -5,8 +5,8 @@
 import logging
 
 import numpy as np
-from .aamp_motifs import aamp_motifs, aamp_match
 
+from .aamp_motifs import aamp_motifs, aamp_match
 from . import core, config
 
 logger = logging.getLogger(__name__)
@@ -23,6 +23,7 @@ def _motifs(
     cutoff,
     max_matches,
     max_motifs,
+    atol=1e-8,
 ):
     """
     Find the top motifs for time series `T`.
@@ -71,10 +72,9 @@ def _motifs(
     max_motifs : int
         The maximum number of motifs to return.
 
-    normalize : bool
-        When set to `True`, this z-normalizes subsequences prior to computing distances.
-        Otherwise, this function gets re-routed to its complementary non-normalized
-        equivalent set in the `@core.non_normalized` function decorator.
+    atol : float, default 1e-8
+        The absolute tolerance parameter. This value will be added to `max_distance`
+        when comparing distances between subsequences.
 
     Return
     ------
@@ -98,7 +98,7 @@ def _motifs(
     candidate_idx = np.argmin(P[-1])
     while len(motif_indices) < max_motifs:
         profile_value = P[-1, candidate_idx]
-        if profile_value > cutoff:  # pragma: no cover
+        if profile_value > cutoff or not np.isfinite(profile_value):  # pragma: no cover
             break
 
         # If max_distance is a constant (independent of the distance profile D of Q
@@ -118,6 +118,8 @@ def _motifs(
             Σ_T=Σ_T,
             max_matches=None,
             max_distance=max_distance,
+            atol=atol,
+            query_idx=candidate_idx,
         )
 
         if len(query_matches) > min_neighbors:
@@ -125,7 +127,7 @@ def _motifs(
             motif_indices.append(query_matches[:max_matches, 1])
 
         for idx in query_matches[:, 1]:
-            core.apply_exclusion_zone(P, int(idx), excl_zone)
+            core.apply_exclusion_zone(P, int(idx), excl_zone, np.inf)
 
         candidate_idx = np.argmin(P[-1])
 
@@ -148,7 +150,9 @@ def motifs(
     cutoff=None,
     max_matches=10,
     max_motifs=1,
+    atol=1e-8,
     normalize=True,
+    p=2.0,
 ):
     """
     Discover the top motifs for time series `T`
@@ -157,6 +161,26 @@ def motifs(
     number of other subsequence matches in `T` (outside the exclusion zone) with a
     distance less or equal to `max_distance`.
 
+    Note that, in the best case scenario, the returned arrays would have shape
+    `(max_motifs, max_matches)` and contain all finite values. However, in reality,
+    many conditions (see below) need to be satisfied in order for this to be true. Any
+    truncation in the number of rows (i.e., motifs)  may be the result of insufficient
+    candidate motifs with matches greater than or equal to `min_neighbors` or that the
+    matrix profile value for the candidate motif was larger than `cutoff`. Similarly,
+    any truncationin in the number of columns (i.e., matches) may be the result of
+    insufficient matches being found with distances (to their corresponding candidate
+    motif) that are equal to or less than `max_distance`. Only motifs and matches that
+    satisfy all of these constraints will be returned.
+
+    If you must return a shape of `(max_motifs, max_matches)`, then you may consider
+    specifying a smaller `min_neighors`, a larger `max_distance`, and/or a larger
+    `cutoff`. For example, while it is ill advised, setting `min_neighbors=1`,
+    `max_distance=np.inf`, and `cutoff=np.inf` will ensure that the shape of the output
+    arrays will be `(max_motifs, max_matches)`. However, given the lack of constraints,
+    the quality of each motif and the quality of each match may be drastically
+    different. Setting appropriate conditions will help ensure appropriately
+    constrained results that may be easier to interpret.
+
     Parameters
     ----------
     T : numpy.ndarray
@@ -194,11 +218,19 @@ def motifs(
     max_motifs : int, default 1
         The maximum number of motifs to return
 
+    atol : float, default 1e-8
+        The absolute tolerance parameter. This value will be added to `max_distance`
+        when comparing distances between subsequences.
+
     normalize : bool, default True
         When set to `True`, this z-normalizes subsequences prior to computing distances.
         Otherwise, this function gets re-routed to its complementary non-normalized
         equivalent set in the `@core.non_normalized` function decorator.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance. This parameter is
+        ignored when `normalize == True`.
+
     Return
     ------
     motif_distances : numpy.ndarray
@@ -213,7 +245,14 @@ def motifs(
 
     See Also
     --------
-    stumpy.match : Find all matches of a query Q in a time series T
+    stumpy.match : Find all matches of a query `Q` in a time series `T`
+    stumpy.mmotifs : Discover the top motifs for the multi-dimensional time series `T`
+    stumpy.stump : Compute the z-normalized matrix profile
+    stumpy.stumped : Compute the z-normalized matrix profile with a distributed dask
+        cluster
+    stumpy.gpu_stump : Compute the z-normalized matrix profile with one or more GPU
+        devices
+    stumpy.scrump : Compute an approximate z-normalized matrix profile
 
     Examples
     --------
@@ -224,6 +263,8 @@ def motifs(
     ...     max_distance=2.0)
     (array([[0.        , 0.11633857]]), array([[0, 4]]))
     """
+    T = core._preprocess(T)
+
     if max_motifs < 1:  # pragma: no cover
         logger.warn(
             "The maximum number of motifs, `max_motifs`, "
@@ -240,7 +281,7 @@ def motifs(
 
     if P.ndim != 1:  # pragma: no cover
         raise ValueError(
-            f"T is {P.ndim}-dimensional and must be 1-dimensional. "
+            f"P is {P.ndim}-dimensional and must be 1-dimensional. "
             "Multidimensional motif discovery is not yet supported."
         )
 
@@ -269,6 +310,7 @@ def motifs(
         cutoff,
         max_matches,
         max_motifs,
+        atol=atol,
     )
 
     return motif_distances, motif_indices
@@ -276,8 +318,8 @@ def motifs(
 
 @core.non_normalized(
     aamp_match,
-    exclude=["normalize", "M_T", "Σ_T", "T_subseq_isfinite", "T_squared"],
-    replace={"M_T": "T_subseq_isfinite", "Σ_T": "T_squared"},
+    exclude=["normalize", "M_T", "Σ_T", "T_subseq_isfinite", "p"],
+    replace={"M_T": "T_subseq_isfinite", "Σ_T": None},
 )
 def match(
     Q,
@@ -286,7 +328,10 @@ def match(
     Σ_T=None,
     max_distance=None,
     max_matches=None,
+    atol=1e-8,
+    query_idx=None,
     normalize=True,
+    p=2.0,
 ):
     """
     Find all matches of a query `Q` in a time series `T`
@@ -324,21 +369,44 @@ def match(
         indices of the most similar `10` subsequences is returned. If `None`, then all
         occurrences are returned.
 
+    atol : float, default 1e-8
+        The absolute tolerance parameter. This value will be added to `max_distance`
+        when comparing distances between subsequences.
+
+    query_idx : int, default None
+        This is the index position along the time series, `T`, where the query
+        subsequence, `Q`, is located.
+        `query_idx` should only be used when the matrix profile is a self-join and
+        should be set to `None` for matrix profiles computed from AB-joins.
+        If `query_idx` is set to a specific integer value, then this will help ensure
+        that the self-match will be returned first.
+
     normalize : bool, default True
         When set to `True`, this z-normalizes subsequences prior to computing distances.
         Otherwise, this function gets re-routed to its complementary non-normalized
         equivalent set in the `@core.non_normalized` function decorator.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance. This parameter is
+        ignored when `normalize == True`.
+
     Returns
     -------
     out : numpy.ndarray
         The first column consists of distances of subsequences of `T` whose distances
-        to `Q` are smaller than `max_distance`, sorted by distance (lowest to highest).
-        The second column consists of the corresponding indices in `T`.
+        to `Q` are less than or equal to `max_distance`, sorted by distance (lowest to
+        highest). The second column consists of the corresponding indices in `T`.
 
     See Also
     --------
-    stumpy.motifs : Discover the top motifs for time series T
+    stumpy.motifs : Discover the top motifs for time series `T`
+    stumpy.mmotifs : Discover the top motifs for the multi-dimensional time series `T`
+    stumpy.stump : Compute the z-normalized matrix profile
+    stumpy.stumped : Compute the z-normalized matrix profile with a distributed dask
+        cluster
+    stumpy.gpu_stump : Compute the z-normalized matrix profile with one or more GPU
+        devices
+    stumpy.scrump : Compute an approximate z-normalized matrix profile
 
     Examples
     --------
@@ -348,8 +416,8 @@ def match(
     ...     )
     array([[0.0011129739290248121, 1]], dtype=object)
     """
-    Q = np.asarray(Q)
-    T = np.asarray(T)
+    Q = core._preprocess(Q)
+    T = core._preprocess(T)
 
     if len(Q.shape) == 1:
         Q = Q[np.newaxis, :]
@@ -378,20 +446,28 @@ def match(
     if M_T is None or Σ_T is None:  # pragma: no cover
         T, M_T, Σ_T = core.preprocess(T, m)
 
-    D = [core.mass(Q[i], T[i], M_T[i], Σ_T[i]) for i in range(d)]
+    D = np.empty((d, n - m + 1))
+    for i in range(d):
+        D[i, :] = core.mass(Q[i], T[i], M_T[i], Σ_T[i])
 
-    D = np.sum(D, axis=0) / d
+    D = np.mean(D, axis=0)
     if not isinstance(max_distance, float):
         max_distance = max_distance(D)
 
     matches = []
 
-    candidate_idx = np.argmin(D)
-    while D[candidate_idx] <= max_distance and len(matches) < max_matches:
-        matches.append([D[candidate_idx], candidate_idx])
-        core.apply_exclusion_zone(D, candidate_idx, excl_zone)
+    if query_idx is not None:
+        candidate_idx = query_idx
+    else:
         candidate_idx = np.argmin(D)
 
-    out = np.array(matches, dtype=object)
+    while (
+        D[candidate_idx] <= atol + max_distance
+        and np.isfinite(D[candidate_idx])
+        and len(matches) < max_matches
+    ):
+        matches.append([D[candidate_idx], candidate_idx])
+        core.apply_exclusion_zone(D, candidate_idx, excl_zone, np.inf)
+        candidate_idx = np.argmin(D)
 
-    return out
+    return np.array(matches, dtype=object)
