@@ -126,6 +126,7 @@ def snippets(
     mpdist_percentage=0.05,
     mpdist_k=None,
     normalize=True,
+    p=2.0,
 ):
     """
     Identify the top `k` snippets that best represent the time series, `T`
@@ -169,6 +170,10 @@ def snippets(
         Otherwise, this function gets re-routed to its complementary non-normalized
         equivalent set in the `@core.non_normalized` function decorator.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance. This parameter is
+        ignored when `normalize == True`.
+
     Returns
     -------
     snippets : numpy.ndarray
